package com.mortals.xhx.module.perform.service.impl;

import com.mortals.xhx.common.code.*;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.module.check.model.CheckReviewRecordEntity;
import com.mortals.xhx.module.check.service.CheckReviewRecordService;
import com.mortals.xhx.module.perform.model.PerformOtherRecordEntity;
import com.mortals.xhx.module.perform.model.PerformRulesEntity;
import com.mortals.xhx.module.perform.service.PerformRulesService;
import com.mortals.xhx.module.perform.service.PerformStaffConfService;
import com.mortals.xhx.module.staff.model.StaffEntity;
import com.mortals.xhx.module.staff.service.StaffService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.perform.dao.PerformReviewRecordDao;
import com.mortals.xhx.module.perform.model.PerformReviewRecordEntity;
import com.mortals.xhx.module.perform.service.PerformReviewRecordService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.ObjectUtils;

import java.util.Date;

/**
 * PerformReviewRecordService
 * 评价差评绩效记录信息 service实现
 *
 * @author zxfei
 * @date 2023-07-11
 */
@Service("performReviewRecordService")
@Slf4j
public class PerformReviewRecordServiceImpl extends AbstractCRUDServiceImpl<PerformReviewRecordDao, PerformReviewRecordEntity, Long> implements PerformReviewRecordService {

    @Autowired
    private CheckReviewRecordService checkReviewRecordService;

    @Autowired
    private PerformRulesService rulesService;
    @Autowired
    private StaffService staffService;
    @Autowired
    private PerformStaffConfService performStaffConfService;

    @Override
    protected void saveBefore(PerformReviewRecordEntity entity, Context context) throws AppException {
        updateStaffRuleNames(entity, context);
    }

    @Override
    protected void updateBefore(PerformReviewRecordEntity entity, Context context) throws AppException {
        updateStaffRuleNames(entity, context);
    }


    private void updateStaffRuleNames(PerformReviewRecordEntity entity, Context context) {
        if (!ObjectUtils.isEmpty(entity.getRuleId())) {
            PerformRulesEntity rulesEntity = rulesService.getCache(entity.getRuleId().toString());
            entity.setRuleName(rulesEntity.getName());
            entity.setSubAddType(rulesEntity == null ? 1 : rulesEntity.getSubAddType());
        }
        if (!ObjectUtils.isEmpty(entity.getStaffId())) {
            StaffEntity staffCache = staffService.getCache(entity.getStaffId().toString());
            entity.setStaffName(staffCache == null ? "" : staffCache.getName());
            entity.setDeptId(staffCache == null ? null : staffCache.getDeptId());
            entity.setDeptName(staffCache == null ? "" : staffCache.getDeptName());
            entity.setSalaId(staffCache == null ? null : staffCache.getSalaId());
            entity.setSalaName(staffCache == null ? "" : staffCache.getSalaName());
        }
        entity.setDeductTime(new Date());
        if(ObjectUtils.isEmpty(entity.getDeductPerson())){
            entity.setDeductPerson(context == null ? "" : context.getUser() == null ? "" : context.getUser().getRealName());
        }
    }

    @Override
    protected void saveAfter(PerformReviewRecordEntity entity, Context context) throws AppException {
        CheckReviewRecordEntity checkReviewRecordEntity = new CheckReviewRecordEntity();
        checkReviewRecordEntity.initAttrValue();
        BeanUtils.copyProperties(entity, checkReviewRecordEntity, BeanUtil.getNullPropertyNames(entity));
        checkReviewRecordEntity.setId(null);
        checkReviewRecordEntity.setRecordId(entity.getId());
        if(performStaffConfService.checkRules(entity.getStaffId(),entity.getDeptId(),entity.getRuleId())){
            checkReviewRecordEntity.setCheckStatus(CheckStatusEnum.已处理.getValue());
            if(checkReviewRecordEntity.getSubAddType()== SubAddTypeEnum.增加.getValue()){
                checkReviewRecordEntity.setCheckResult(CheckResultAddEnum.核查加分.getValue());
            }else {
                checkReviewRecordEntity.setCheckResult(CheckResultSubEnum.核查扣分.getValue());
            }
            checkReviewRecordEntity.setUpdateTime(new Date());
        }else {
            checkReviewRecordEntity.setCheckStatus(CheckStatusEnum.未处理.getValue());      //非自动扣分相设置为未处理
        }
        checkReviewRecordService.save(checkReviewRecordEntity, context);
    }

    @Override
    public void updateProcessStatus(Long id, Integer status) throws AppException {
        if (id == null) {
            throw new AppException("评价差评绩效记录Id不能为空");
        }
        PerformReviewRecordEntity entity = this.get(id);
        if (entity == null) {
            throw new AppException("无效的评价差评绩效记录Id");
        }
        if (status == null) {
            status = 1; //处理状态(1.未核查,2.已核查)
        }
        PerformReviewRecordEntity update = new PerformReviewRecordEntity();
        update.setId(id);
        update.setProcessStatus(status);
        update.setUpdateTime(new Date());
        dao.update(update);
    }

}