package com.mortals.xhx.module.perform.web;

import cn.hutool.core.date.DateUtil;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.OrderCol;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.base.system.user.model.UserQuery;
import com.mortals.xhx.base.system.user.service.UserService;
import com.mortals.xhx.common.code.*;
import com.mortals.xhx.module.dept.model.DeptQuery;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.perform.model.PerformRulesQuery;
import com.mortals.xhx.module.perform.model.vo.AppealInfo;
import com.mortals.xhx.module.perform.model.vo.AppealSummaryQuery;
import com.mortals.xhx.module.perform.service.PerformRulesService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.module.perform.model.PerformAttendAppealEntity;
import com.mortals.xhx.module.perform.service.PerformAttendAppealService;
import org.apache.commons.lang3.ArrayUtils;
import com.mortals.framework.util.StringUtils;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONObject;

import java.util.Arrays;

import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import static com.mortals.framework.ap.SysConstains.*;
import static java.util.stream.Collectors.toMap;

/**
 * 绩效记录申诉信息
 *
 * @author zxfei
 * @date 2023-07-12
 */
@RestController
@RequestMapping("perform/attend/appeal")
@Slf4j
public class PerformAttendAppealController extends BaseCRUDJsonBodyMappingController<PerformAttendAppealService, PerformAttendAppealEntity, Long> {

    @Autowired
    private ParamService paramService;
    @Autowired
    private UserService userService;
    @Autowired
    private PerformRulesService rulesService;
    @Autowired
    private DeptService deptService;


    public PerformAttendAppealController() {
        super.setModuleDesc("绩效记录申诉信息");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {

        this.addDict(model, "updateUserId", userService.find(new UserQuery()).stream().collect(toMap(x -> x.getId().toString(), y -> y.getRealName(), (o, n) -> n)));
        this.addDict(model, "ruleId", rulesService.find(new PerformRulesQuery()).stream().collect(toMap(x -> x.getId().toString(), y -> y.getName(), (o, n) -> n)));
        this.addDict(model, "checkResult", CheckResultEnum.getEnumMap());
        //this.addDict(model, "checkResult", CheckResultAddEnum.getEnumMap());

        this.addDict(model, "subMethod", SubMethodEnum.getEnumMap());
        this.addDict(model, "subAddType", SubAddTypeEnum.getEnumMap());
        this.addDict(model, "processStatus", ProcessStatusEnum.getEnumMap());
        this.addDict(model, "appealResult", AppealResultEnum.getEnumMap());
        this.addDict(model, "reviewResult", paramService.getParamBySecondOrganize("PerformAttendAppeal", "reviewResult"));
        this.addDict(model, "reviewSource", paramService.getParamBySecondOrganize("PerformAttendAppeal", "reviewSource"));
        this.addDict(model, "irregularType", IrregularTypeEnum.getEnumMap());
        this.addDict(model, "irregularOtherType", IrregularOtherTypeEnum.getEnumMap());
        this.addDict(model, "deptId", deptService.find(new DeptQuery()).stream().collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getDeptName())));

        super.init(model, context);
    }


    @Override
    protected void doListBefore(PerformAttendAppealEntity query, Map<String, Object> model, Context context) throws AppException {
        if (ObjectUtils.isEmpty(query.getOrderColList())) {
            query.setOrderColList(Arrays.asList(new OrderCol("appealTime", OrderCol.DESCENDING)));
        } else {
            query.getOrderColList().add(new OrderCol("appealTime", OrderCol.DESCENDING));
        }
        if(ObjectUtils.isEmpty(query.getAppealTimeStart())){
            LocalDateTime currentDate = LocalDateTime.now();
            LocalDateTime firstDayOfMonth = currentDate.withDayOfMonth(1);
            query.setAppealTimeStart(DateUtil.format(firstDayOfMonth,"yyyy-MM-dd"));
            //query.setAppealTimeEnd(DateUtil.formatDate(query.getCreateTime()));
        }
    }

    /**
     * 申诉统计
     */
    @PostMapping(value = "stat")
    public String appealStat(@RequestBody PerformAttendAppealEntity appeal) {
        JSONObject jsonObject = new JSONObject();
        String busiDesc = this.getModuleDesc() + "统计";
        try {
            Rest<List<AppealInfo>> rest = this.service.stat(appeal, getContext());
            if (rest.getCode() == YesNoEnum.NO.getValue()) {
                throw new AppException("统计异常！");
            } else {
                jsonObject.put(KEY_RESULT_DATA, rest.getData());
            }
            recordSysLog(request, busiDesc + " 【成功】");
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
        } catch (Exception e) {
            log.error("申诉审核", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }


    /**
     * 申诉审核
     */
    @PostMapping(value = "audit")
    public String appealAudit(@RequestBody PerformAttendAppealEntity appeal) {
        log.info("appeal audit ==>{}", JSONObject.toJSONString(appeal));
        JSONObject jsonObject = new JSONObject();
        String busiDesc = this.getModuleDesc() + "审核";
        try {
            Rest<Void> audit = this.service.audit(appeal, getContext());
            if (audit.getCode() == YesNoEnum.NO.getValue()) {
                throw new AppException("申诉审核异常！");
            }
            recordSysLog(request, busiDesc + " 【成功】");
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            jsonObject.put(KEY_RESULT_MSG, busiDesc + "成功!");
        } catch (Exception e) {
            log.error("申诉审核", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }

    /**
     * 申述分布情况统计
     *
     * @param query
     * @return
     */
    @PostMapping({"perform/summary"})
    @UnAuth
    public Rest<Object> getAppealSummaryList(@RequestBody AppealSummaryQuery query) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查询申述分布情况统计";
        int code = 1;
        try {
            List<PerformAttendAppealEntity> result = this.getService().getAppealSummaryList(query);
            Map<String, String> dict = new HashMap<>();
            //attend:考勤绩效,review:评价差评绩效,complain:评价投诉绩效,gowork:办件绩效,effect:效能绩效,other:其它绩效
            dict.put("attend", "考勤绩效");
            dict.put("review", "评价差评绩效");
            dict.put("complain", "评价投诉绩效");
            dict.put("gowork", "办件绩效");
            dict.put("effect", "效能绩效");
            dict.put("other", "其它绩效");
            this.addDict(model, "performType", dict);
            model.put("data", result);
            model.put("message_info", busiDesc + "成功");
            if (!ObjectUtils.isEmpty(context) && !ObjectUtils.isEmpty(context.getUser())) {
                this.recordSysLog(this.request, busiDesc + " 【成功】");
            }
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }
        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.remove("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }

}