package com.mortals.xhx.module.perform.web;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.OrderCol;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.base.system.user.model.UserQuery;
import com.mortals.xhx.base.system.user.service.UserService;
import com.mortals.xhx.common.code.*;
import com.mortals.xhx.module.attendance.model.AttendanceRecordHikQuery;
import com.mortals.xhx.module.dept.model.DeptQuery;
import com.mortals.xhx.module.dept.service.DeptService;
import com.mortals.xhx.module.perform.model.PerformEffectRecordQuery;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StopWatch;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.module.perform.model.PerformEffectRecordEntity;
import com.mortals.xhx.module.perform.service.PerformEffectRecordService;
import org.apache.commons.lang3.ArrayUtils;
import com.mortals.framework.util.StringUtils;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONObject;

import java.util.Arrays;

import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import static com.mortals.framework.ap.SysConstains.*;

/**
 * 效能绩效记录信息
 *
 * @author zxfei
 * @date 2023-07-11
 */
@RestController
@RequestMapping("perform/effect/record")
@Slf4j
public class PerformEffectRecordController extends BaseCRUDJsonBodyMappingController<PerformEffectRecordService, PerformEffectRecordEntity, Long> {

    @Autowired
    private ParamService paramService;
    @Autowired
    private UserService userService;
    @Autowired
    private DeptService deptService;

    public PerformEffectRecordController() {
        super.setModuleDesc("效能绩效记录信息");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "irregularType", paramService.getParamBySecondOrganize("PerformEffectRecord", "irregularType"));
        this.addDict(model, "processStatus", ProcessStatusEnum.getEnumMap());
        this.addDict(model, "checkStatus", CheckStatusEnum.getEnumMap());
        this.addDict(model, "subMethod", SubMethodEnum.getEnumMap());
        this.addDict(model, "subAddType", SubAddTypeEnum.getEnumMap());
        this.addDict(model, "checkType", CheckTypeEnum.getEnumMap());
        this.addDict(model, "deptId", deptService.find(new DeptQuery()).stream().collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getDeptName())));

        Map<String, String> collect = userService.find(new UserQuery(), getContext()).stream()
                .collect(Collectors.toMap(x -> x.getId().toString(), y -> y.getRealName(), (o, n) -> n));

        this.addDict(model, "createUserId", collect);
        super.init(model, context);
    }

    @Override
    protected void doListBefore(PerformEffectRecordEntity query, Map<String, Object> model, Context context) throws AppException {
        if (ObjectUtils.isEmpty(query.getOrderColList())) {
            query.setOrderColList(Arrays.asList(new OrderCol("happenTime", OrderCol.DESCENDING)));
        } else {
            query.getOrderColList().add(new OrderCol("happenTime", OrderCol.DESCENDING));
        }

        if (ObjectUtils.isEmpty(query.getHappenTimeStart())) {
            LocalDateTime currentDate = LocalDateTime.now();
            LocalDateTime firstDayOfMonth = currentDate.withDayOfMonth(1);
            query.setHappenTimeStart(DateUtil.format(firstDayOfMonth, "yyyy-MM-dd"));
            //query.setHappenTimeEnd(DateUtil.formatDate(query.getCreateTime()));
        }
    }


    @PostMapping(value = "/sync")
    @UnAuth
    public Rest<String> syncEffectRecord(@RequestBody PerformEffectRecordQuery query) {
        Rest<String> ret = new Rest();
        Map<String, Object> model = new HashMap();
        Context context = this.getContext();
        String busiDesc = "查询" + this.getModuleDesc();
        int code = 1;
        try {
            //天数区间分段计算
            DateTime attendStart = DateUtil.parseDate(query.getHappenTimeStart());
            DateTime attendEnd = DateUtil.parseDate(query.getHappenTimeEnd());
            Long compare = DateUtil.between(attendEnd, attendStart, DateUnit.DAY);
            StopWatch stopWatch = new StopWatch("stopwatch attend1");
            log.info("效能同步计算天数区间：{}", compare);
            for (int i = 0; i <= compare.intValue(); i++) {
                DateTime curDate = DateUtil.offsetDay(attendStart, i);
                log.info("效能同步记录日期：{}", curDate.toDateStr());
                stopWatch.start("执行本地方法");
                PerformEffectRecordQuery performEffectRecordQuery = new PerformEffectRecordQuery();
                performEffectRecordQuery.setHappenTimeStart(curDate.toDateStr());
                performEffectRecordQuery.setHappenTimeEnd(curDate.toDateStr());
                this.service.syncEffectRecord(performEffectRecordQuery, context);
                this.service.syncEffectRecordImgs(performEffectRecordQuery, context);
                stopWatch.stop();
                log.info("效能同步记录日期：{} 完成,耗时：{}ms", curDate.toDateStr(), stopWatch.getLastTaskTimeMillis());
            }
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            code = -1;
            this.doException(this.request, busiDesc, model, e);
            model.put("message_info", e.getMessage());
        }

        this.init(model, context);
        ret.setCode(code);
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;

    }
}